// Prevents additional console window on Windows in release, DO NOT REMOVE!!
#![cfg_attr(not(debug_assertions), windows_subsystem = "windows")]

use std::{path::PathBuf, sync::Mutex};

use config::Config;
use crossbeam::channel::Sender;
use overlay::{Event, LockedOverlayData, Overlay};

use plan::Plan;
use poe_data::world_area::WorldAreasMap;
use simple_logger::SimpleLogger;
use storage::Storage;
use tauri::AppHandle;
use tauri::CustomMenuItem;
use tauri::Manager;

use tauri::SystemTray;
use tauri::SystemTrayEvent;
use tauri::SystemTrayMenu;
use tauri::SystemTrayMenuItem;

mod config;
mod overlay;
mod plan;
mod storage;

#[tauri::command]
fn set_interactable(interactable: bool, state: tauri::State<Sender<Event>>) {
    log::info!("set_interactable: {interactable:?}");
    if interactable {
        state.send(overlay::State::Interactable {}.into()).ok();
    } else {
        state.send(overlay::Event::Return).ok();
    }
}

#[tauri::command]
fn load_world_areas() -> WorldAreasMap {
    log::info!("Loading world areas");
    poe_data::world_area::load_world_areas_map(include_str!(
        "../../data/processed_world_areas.json"
    ))
}

#[tauri::command]
fn load_config(state: tauri::State<Mutex<Storage>>) -> Option<Config> {
    Some(state.lock().ok()?.config.clone())
}

#[tauri::command]
fn set_config(
    config: Config,
    state: tauri::State<Mutex<Storage>>,
    window: tauri::Window,
    app: AppHandle,
) {
    log::info!("Saved config: {:?}", config);
    if let Ok(mut storage) = state.lock() {
        storage.config = config.clone();
        storage.save();

        let other_window = if window.label() == "Overlay" {
            "Normal"
        } else {
            "Overlay"
        };
        //Ignore failures! woohoo
        // app.emit_to(other_window, "config", config).ok();
    }
}

#[tauri::command]
fn load_plan(path: PathBuf, state: tauri::State<Mutex<Storage>>) -> Option<Plan> {
    if let Ok(mut storage) = state.lock() {
        if let Some(path_string) = path.to_str() {
            storage.last_plan = Some(path_string.to_string());
            let plan = Storage::load_plan(storage.last_plan.as_ref()?);
            log::info!("got plan: {plan:?}");
            return plan;
        }
    }
    None
}

#[tauri::command]
fn save_plan(path: PathBuf, plan: Plan) -> bool {
    if let Some(path_string) = path.with_extension("json").to_str() {
        return Storage::save_plan(path_string, plan).is_ok();
    }

    false
}

#[tauri::command]
fn load_stored_plans() -> Vec<String> {
    Storage::enumerate_plans()
}

fn main() {
    SimpleLogger::new()
        .with_module_level("underlayer", log::LevelFilter::Info)
        .with_level(log::LevelFilter::Trace)
        .init()
        .expect("Could not init logger");

    let settings = CustomMenuItem::new("settings".to_string(), "Settings");
    let editor = CustomMenuItem::new("editor".to_string(), "Plan Editor");
    let exit = CustomMenuItem::new("exit".to_string(), "Exit");
    let tray_menu = SystemTrayMenu::new()
        .add_item(settings)
        .add_item(editor)
        .add_native_item(SystemTrayMenuItem::Separator)
        .add_item(exit);
    let system_tray = SystemTray::new().with_menu(tray_menu);

    tauri::Builder::default()
        .setup(|app| {
            let tx = Overlay::initialize(
                app.get_window("Overlay")
                    .expect("Could not get main overlay window"),
                "Untitled 1 - Mousepad",
            );
            app.manage(tx);
            app.manage(Mutex::new(Storage::default()));

            app.get_window("Overlay")
                .expect("Could not get main overlay window")
                .open_devtools();

            Ok(())
        })
        .invoke_handler(tauri::generate_handler![
            set_interactable,
            load_world_areas,
            load_config,
            set_config,
            load_plan,
            load_stored_plans,
            save_plan
        ])
        .system_tray(system_tray)
        .on_system_tray_event(|app, event| match event {
            SystemTrayEvent::MenuItemClick { id, .. } => match id.as_str() {
                "exit" => {
                    std::process::exit(0);
                }
                "editor" | "settings" => {
                    if let Some(window) = app.get_window("Normal") {
                        window.show().ok();
                    }
                }
                _ => {}
            },
            _ => {}
        })
        .run(tauri::generate_context!())
        .expect("error while running tauri application");
}
